from rria_api.gen3.api_gen3.device_connection import DeviceConnection


class ConnectGen3:
    route = None

    def __init__(self, ip_address, credentials):
        self.ip_address = ip_address
        self.credentials = credentials

        self.connect_instance = DeviceConnection(ip_address=self.ip_address,
                                                 credentials=(credentials[0], credentials[1]))

    def connect_robot(self):
        self.route = self.connect_instance.connect()

        return self.route

    def disconnect_robot(self):
        self.connect_instance = self.connect_instance.disconnect()



import math


class ActionNed:
    def __init__(self, robot_object):
        self.robot_object = robot_object

    def move_joints(self, j1, j2, j3, j4, j5, j6):
        trans = math.pi / 180
        self.robot_object.move_joints(j1 * trans, j2 * trans, j3 * trans, j4 * trans, j5 * trans, j6 * trans)

    def get_joints(self):
        joints_rad = self.robot_object.get_joints()
        joints_degrees = [round(joint * 180 / math.pi, 3) for joint in joints_rad]
        return joints_degrees

    def move_cartesian(self, x, y, z, roll, pitch, yaw):
        trans = math.pi / 180
        self.robot_object.move_pose(x, y, z, roll * trans, pitch * trans, yaw * trans)

    def get_cartesian(self):
        return self.robot_object.get_pose()

    def move_to_home(self):
        self.robot_object.move_to_home_pose()

    def move_to_zero(self):
        self.robot_object.move_joints([0.0, 0.0, 0.0, 0.0, 0.0, 0.0])

    def open_gripper(self):
        self.robot_object.release_with_tool()

    def close_gripper(self):
        self.robot_object.grasp_with_tool()

    def set_velocity(self, velocity):
        self.robot_object.set_arm_max_velocity(velocity)

    def calibrate(self):
        self.robot_object.calibrate_auto()

    def go_to_sleep(self):
        self.robot_object.go_to_sleep()



from pyniryo import *


class ConnectNed:
    def __init__(self):
        self.robot_object = None

    def connect_robot(self, ip_address):
        self.robot_object = NiryoRobot(ip_address)
        self.robot_object.calibrate_auto()
        return self.robot_object

    def disconnect_robot(self):
        self.robot_object.close_connection()



from kortex_api.autogen.client_stubs.BaseCyclicClientRpc import BaseCyclicClient
from kortex_api.autogen.client_stubs.BaseClientRpc import BaseClient
from rria_api.gen3.api_gen3.gen3_api import Gen3Api


class ActionGen3:
    def __init__(self, route):
        self.route = route
        self.base = BaseClient(self.route)
        self.base_cyclic = BaseCyclicClient(self.route)

    def move_joints(self, joints_list):
        Gen3Api().angular_movement(self.base, joints_list)

    def get_joints(self):
        return Gen3Api().get_joints(self.base_cyclic)

    def move_cartesian(self, cartesian_list):
        Gen3Api().cartesian_movement(self.base, cartesian_list)

    def get_cartesian(self):
        return Gen3Api().get_cartesian(self.base_cyclic)

    def move_to_home(self):
        Gen3Api().move_to_home(self.base)

    def move_to_zero(self):
        joints_list = [0, 0, 0, 0, 0, 0]
        Gen3Api().angular_movement(self.base, joints_list)

    def open_gripper(self):
        Gen3Api().open_gripper(self.base, 2)

    def close_gripper(self):
        Gen3Api().close_gripper(self.base, 2)

    def set_velocity(self, velocity):
        Gen3Api().set_velocity(self.base, velocity)

    def calibrate(self):
        ...

    def go_to_sleep(self):
        ...



from kortex_api.autogen.messages import Base_pb2
import threading
import time

TIMEOUT_DURATION = 10


# Classe de movimentação adaptada dos exemplos disponibilizados pela API.
# Essa classe é a única (até agora) que tem contato direto com a base (CPU) do Gen3
class Gen3Api:
    @staticmethod
    def check_for_end_or_abort(error):
        def check(notification, e=error):
            if notification.action_event == Base_pb2.ACTION_END or notification.action_event == Base_pb2.ACTION_ABORT:
                e.set()

        return check

    def move_to_home(self, base):
        # Make sure the arm is in Single Level Servoing mode
        base_servo_mode = Base_pb2.ServoingModeInformation()
        base_servo_mode.servoing_mode = Base_pb2.SINGLE_LEVEL_SERVOING
        base.SetServoingMode(base_servo_mode)

        # Move arm to ready position
        action_type = Base_pb2.RequestedActionType()
        action_type.action_type = Base_pb2.REACH_JOINT_ANGLES
        action_list = base.ReadAllActions(action_type)
        action_handle = None
        for action in action_list.action_list:
            if action.name == "Home":
                action_handle = action.handle

        if action_handle is None:
            print("Can't reach safe position. Exiting")
            return False

        e = threading.Event()
        notification_handle = base.OnNotificationActionTopic(
            self.check_for_end_or_abort(e),
            Base_pb2.NotificationOptions()
        )

        base.ExecuteActionFromReference(action_handle)
        finished = e.wait(TIMEOUT_DURATION)
        base.Unsubscribe(notification_handle)

        if finished:
            print("Safe position reached")
        else:
            print("Timeout on action notification wait")
        return finished

    @staticmethod
    def populate_angular_pose(jointPose, durationFactor):
        waypoint = Base_pb2.AngularWaypoint()
        waypoint.angles.extend(jointPose)
        waypoint.duration = durationFactor * 5.0

        return waypoint

    # Método utilizado para mover o robô para waypoints definidos.
    def move_trajectory(self, base, joints_list):
        base_servo_mode = Base_pb2.ServoingModeInformation()
        base_servo_mode.servoing_mode = Base_pb2.SINGLE_LEVEL_SERVOING
        base.SetServoingMode(base_servo_mode)

        joint_poses = joints_list

        waypoints = Base_pb2.WaypointList()
        waypoints.duration = 0.0
        waypoints.use_optimal_blending = False

        index = 0
        for joint_pose in joint_poses:
            waypoint = waypoints.waypoints.add()
            waypoint.name = "waypoint_" + str(index)
            durationFactor = 1
            # Joints/motors 5 and 7 are slower and need more time
            if index == 4 or index == 6:
                durationFactor = 6  # Min 30 seconds

            waypoint.angular_waypoint.CopyFrom(self.populate_angular_pose(joint_pose, durationFactor))
            index = index + 1

            # Verify validity of waypoints
        result = base.ValidateWaypointList(waypoints)
        if len(result.trajectory_error_report.trajectory_error_elements) == 0:

            e = threading.Event()
            notification_handle = base.OnNotificationActionTopic(
                self.check_for_end_or_abort(e),
                Base_pb2.NotificationOptions()
            )

            base.ExecuteWaypointTrajectory(waypoints)

            finished = e.wait(TIMEOUT_DURATION)
            base.Unsubscribe(notification_handle)

            if finished:
                pass
            else:
                print("Timeout on action notification wait")
            return finished
        else:
            pass

    # O método move_joints chama esse método para mover as juntas.
    def angular_movement(self, base, joints_list):
        # Starting angular action movement
        action = Base_pb2.Action()
        action.name = "Angular action movement"
        action.application_data = ""

        # Place arm straight up
        joint_id = 1
        for joint_value in joints_list:
            joint_angle = action.reach_joint_angles.joint_angles.joint_angles.add()
            joint_angle.joint_identifier = joint_id
            joint_angle.value = joint_value
            joint_id += 1

        e = threading.Event()
        notification_handle = base.OnNotificationActionTopic(
            self.check_for_end_or_abort(e),
            Base_pb2.NotificationOptions()
        )

        # Executing action
        base.ExecuteAction(action)

        # Waiting for movement to finish
        finished = e.wait(TIMEOUT_DURATION)
        base.Unsubscribe(notification_handle)

        if finished:
            pass
        else:
            print("Timeout on action notification wait")
        return finished

    def cartesian_movement(self, base, cartesian_list):
        # Starting Cartesian action movement
        action = Base_pb2.Action()
        action.name = "Example Cartesian action movement"
        action.application_data = ""

        cartesian_pose = action.reach_pose.target_pose
        cartesian_pose.x = cartesian_list[0]  # (meters)
        cartesian_pose.y = cartesian_list[1]  # (meters)
        cartesian_pose.z = cartesian_list[2]  # (meters)
        cartesian_pose.theta_x = cartesian_list[3]  # (degrees)
        cartesian_pose.theta_y = cartesian_list[4]  # (degrees)
        cartesian_pose.theta_z = cartesian_list[5]  # (degrees)

        e = threading.Event()
        notification_handle = base.OnNotificationActionTopic(
            self.check_for_end_or_abort(e),
            Base_pb2.NotificationOptions()
        )

        # Executing action
        base.ExecuteAction(action)

        # Waiting for movement to finish
        finished = e.wait(TIMEOUT_DURATION)
        base.Unsubscribe(notification_handle)

        if finished:
            pass
        else:
            print("Timeout on action notification wait")
        return finished

    # Função para fechar a garra. Ver outra opção ao sleep.
    @staticmethod
    def close_gripper(base, close_time):
        # Create the GripperCommand we will send
        gripper_command = Base_pb2.GripperCommand()
        finger = gripper_command.gripper.finger.add()

        # Close the gripper with position increments
        gripper_command.mode = Base_pb2.GRIPPER_POSITION
        finger.finger_identifier = 1
        finger.value = 1
        base.SendGripperCommand(gripper_command)

        time.sleep(close_time)

    # Função para abrir a garra. Ver outra opção ao sleep.
    @staticmethod
    def open_gripper(base, open_time):
        # Create the GripperCommand we will send
        gripper_command = Base_pb2.GripperCommand()
        finger = gripper_command.gripper.finger.add()

        # Close the gripper with position increments
        gripper_command.mode = Base_pb2.GRIPPER_POSITION
        finger.finger_identifier = 1
        finger.value = 0
        base.SendGripperCommand(gripper_command)

        time.sleep(open_time)

    @staticmethod
    def get_joints(base_cyclic):
        feedback = base_cyclic.RefreshFeedback()
        joints = [round(feedback.actuators[joint].position, 3) for joint in range(0, 6)]

        return joints

    @staticmethod
    def get_cartesian(base_cyclic):
        fb = base_cyclic.RefreshFeedback()
        pose_meters = [round(fb.tool_pose_x, 3), round(fb.tool_pose_y, 3), round(fb.tool_pose_z, 3),
                       round(fb.tool_pose_theta_x, 3), round(fb.tool_pose_theta_y, 3), round(fb.tool_pose_theta_z, 3)]

        return pose_meters

    @staticmethod
    def set_velocity(base, velocity):
        ...



from rria_api.ned.connect_ned import ConnectNed
from rria_api.ned.action_ned import ActionNed
from rria_api.gen3.connect_gen3 import ConnectGen3
from rria_api.gen3.action_gen3 import ActionGen3
from time import sleep


class RobotObject:
    def __init__(self, ip_address, robot_type):
        self.ip_address = ip_address
        self.robot_type = robot_type

        # This atribute is used to store the general robot instance
        self.robot_instance = None

        # This atribute is used to store the general connection instance
        self.connection_instance = None

    def connect_robot(self):
        """
        Connect robot depends on the robot type
        :rtype: bool

        """

        if self.robot_type == 'gen3':
            try:
                self.connection_instance = ConnectGen3(self.ip_address, ["admin", "admin"])
                self.robot_instance = self.connection_instance.connect_robot()

                return True

            except(Exception,):
                print('The connection attempt failed. Check the physical connection to the robot and try again later.')

                return False

        if self.robot_type == 'ned':
            try:
                self.connection_instance = ConnectNed()
                self.robot_instance = self.connection_instance.connect_robot(self.ip_address)

                return True

            except(Exception,):
                print('The connection attempt failed. Check the physical connection to the robot and try again later.')

                return False

        if self.robot_type == 'denso':
            pass

        if self.robot_type == 'test':
            sleep(1)
            return True

    def disconnect_robot(self):
        """
        Close connection with robot
        :rtype: None

        """
        if self.robot_type == 'gen3':
            self.connection_instance.disconnect_robot()

        if self.robot_type == 'ned':
            self.connection_instance.disconnect_robot()

        if self.robot_type == 'denso':
            pass

        if self.robot_type == 'test':
            sleep(1)
            return True

    def safe_disconnect(self):
        """
        Move robot for home position and close connection with robot. Home position dependes on robot type. For Gen3 is
        [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]  degrees and for Ned is [0.0, 0.3, -1.3, 0.0, 0.0, 0.0] radians.
        :rtype: None

        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).move_to_zero()
            self.connection_instance.disconnect_robot()

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).move_to_home()
            self.connection_instance.disconnect_robot()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    # Move Joints/Cartesian methods
    def joints(self):
        return self.get_joints()

    def get_joints(self):
        """
        Get joints value in radians
        You can also use a getter ::

            joints = robot.get_joints()
            joints = robot.joints

        :return: List of joints value
        :rtype: list[float]
        """
        if self.robot_type == 'gen3':
            return ActionGen3(self.robot_instance).get_joints()

        if self.robot_type == 'ned':
            return ActionNed(self.robot_instance).get_joints()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(0.5)
            return ['J1', 'J2', 'J3', 'J4', 'J5', 'J6']

    def move_joints(self, j1, j2, j3, j4, j5, j6):
        """
        Move robot joints. Joints are expressed in degrees.

        All lines of the next example realize the same operation: ::

            robot.move_joints(0.2, 0.1, 0.3, 0.0, 0.5, 0.0)

        :param j1: joint 1,
        :type j1: float
        :param j2: joint 2,
        :type j2: float
        :param j3: joint 3,
        :type j3: float
        :param j4: joint 4,
        :type j4: float
        :param j5: joint 5,
        :type j5: float
        :param j6: joint 6,
        :type j6: float
        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).move_joints([j1, j2, j3, j4, j5, j6])

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).move_joints(j1, j2, j3, j4, j5, j6)

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def cartesian(self):
        return self.get_cartesian()

    def get_cartesian(self):
        """
        Get end effector link pose as [x, y, z, roll, pitch, yaw].
        x, y & z are expressed in meters / roll, pitch & yaw are expressed in radians
        You can also use a getter ::

            pose = robot.get_pose()
            pose = robot.pose

        :rtype: PoseObject
        """
        if self.robot_type == 'gen3':
            return ActionGen3(self.robot_instance).get_cartesian()

        if self.robot_type == 'ned':
            return ActionNed(self.robot_instance).get_cartesian()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def move_cartesian(self, x, y, z, roll, pitch, yaw):
        """
        Move robot end effector pose to a (x, y, z, roll, pitch, yaw, frame_name) pose
        in a particular frame (frame_name) if defined.
        x, y & z are expressed in meters / roll, pitch & yaw are expressed in radians

        All lines of the next example realize the same operation: ::

            robot.move_cartesian(0.2, 0.1, 0.3, 0.0, 0.5, 0.0)

        :param x: coordinate x,
        :type x: float
        :param y: coordinate y,
        :type y: float
        :param z: coordinate z,
        :type z: float
        :param roll: rotation on x-axis,
        :type roll: float
        :param pitch: rotation on y-axis,
        :type pitch: float
        :param yaw: rotation on z-axis,
        :type yaw: float
        :rtype: None
        """
        if self.robot_type == 'gen3':
            return ActionGen3(self.robot_instance).move_cartesian([x, y, z, roll, pitch, yaw])

        if self.robot_type == 'ned':
            return ActionNed(self.robot_instance).move_cartesian(x, y, z, roll, pitch, yaw)

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    # TODO: Pegar os valores de home do robot
    def move_to_home(self):
        """
        Move robot for home position. Home position dependes on robot type. For Gen3 is [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        degrees and for Ned is [0.0, 0.3, -1.3, 0.0, 0.0, 0.0] radians.

        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).move_to_home()

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).move_to_home()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def move_to_zero(self):
        """
        Move robot for zero position. Home position dependes on robot type. For Gen3 is [0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
        degrees and for Ned is [0.0, 0.0, 0.0, 0.0, 0.0, 0.0] radians.

        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).move_to_zero()

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).move_to_zero()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def open_gripper(self):
        """
        Open gripper.
        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).open_gripper()

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).open_gripper()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def close_gripper(self):
        """
        Close gripper
        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).close_gripper()

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).close_gripper()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    # TODO: Ver a função de aumento de velocidade para o Gen3
    def set_velocity(self, velocity):
        """
        Limit arm max velocity to a percentage of its maximum velocity. For niryo one, velocity is a percentage of 100.
        For gen3, there are two types of velocities, an angular and a cartesian. The speed used in this method is
        angular.

        :param velocity: Should be between 1 & 100 for niryo
        :type velocity: int
        :rtype: None
        """
        if self.robot_type == 'gen3':
            ActionGen3(self.robot_instance).set_velocity(velocity)

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).set_velocity(velocity)

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def calibrate(self):
        """
        Start an automatic motors calibration if motors are not calibrated yet

        :rtype: None
        """
        if self.robot_type == 'gen3':
            print('Gen3 NÃO necessita de calibração')

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).calibrate()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True

    def go_to_sleep(self):
        """
        Go home pose and activate learning mode. The function is avaliable only for Ned robot.

        :rtype: None
        """
        if self.robot_type == 'gen3':
            ...

        if self.robot_type == 'ned':
            ActionNed(self.robot_instance).go_to_sleep()

        if self.robot_type == 'denso':
            ...

        if self.robot_type == 'test':
            sleep(1)
            return True



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rria_api', 'rria_api.gen3', 'rria_api.gen3.api_gen3', 'rria_api.ned']

package_data = \
{'': ['*']}

install_requires = \
['pyniryo>=1.1.2,<2.0.0']

setup_kwargs = {
    'name': 'rria-api',
    'version': '1.0.2',
    'description': '',
    'long_description': '# robot-api\n Robot API\n\nLinks Úteis:\nhttps://docs.niryo.com/dev/ros/v4.1.1/en/source/overview.html',
    'author': 'felipeadsm',
    'author_email': '97059009+felipeadsm@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from kortex_api.RouterClient import RouterClient, RouterClientSendOptions
from kortex_api.SessionManager import SessionManager
from kortex_api.autogen.messages import Session_pb2
from kortex_api.TCPTransport import TCPTransport


class DeviceConnection:
    TCP_PORT = 10000

    def __init__(self, ip_address, port=TCP_PORT, credentials=("", "")):
        self.ip_address = ip_address
        self.port = port
        self.credentials = credentials

        self.sessionManager = None

        # Setup API
        self.transport = TCPTransport()
        self.router = RouterClient(self.transport, RouterClient.basicErrorCallback)

    def connect(self):
        self.transport.connect(self.ip_address, self.port)

        session_info = Session_pb2.CreateSessionInfo()
        session_info.username = self.credentials[0]
        session_info.password = self.credentials[1]
        session_info.session_inactivity_timeout = 10000  # (milliseconds)
        session_info.connection_inactivity_timeout = 2000  # (milliseconds)

        self.sessionManager = SessionManager(self.router)
        print("Logging as", self.credentials[0], "on device", self.ip_address)
        self.sessionManager.CreateSession(session_info)

        return self.router

    def disconnect(self):
        if self.sessionManager is not None:
            router_options = RouterClientSendOptions()
            router_options.timeout_ms = 1000

            self.sessionManager.CloseSession(router_options)

        self.transport.disconnect()
